%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This work is licensed under a Creative Commons Attribution 4.0
% International License. http://creativecommons.org/licenses/by/4.0/
% The authors do not assume liability for errors contained in or for
% damages arising from the use of the software.
%
% CC-BY (4.0) Oliver Taubmann (oliver.taubmann@fau.de), 2015
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [prof_ps, ps] = create_profiles(xs, ys, prof_length, prof_spacing)
% Create profiles for segmentation (xs, ys)
% Profile length/spacing are optional parameters
% The rows of prof_ps contain the start and end 
% points of the profiles such that
% prof_ps(1, :) -- prof_ps(2, :) is the first profile,
% prof_ps(3, :) -- prof_ps(4, :) the second, etc.
% The rows of ps are the points at which the profiles intersect
% the segmentation.

if ~exist('prof_length', 'var') || isempty(prof_length)
  prof_length = 10;
end

if ~exist('prof_spacing', 'var') || isempty(prof_spacing)
  prof_spacing = 2;
end

% Compute total length of segmentation
len = sum(sqrt((xs(2:end) - xs(1:end-1)).^2 + (ys(2:end) - ys(1:end-1)).^2));

num_profiles = round(len / prof_spacing);

prof_ps = zeros(2*num_profiles, 2);
ps      = zeros(  num_profiles, 2);

for i = 1:num_profiles
    
    pp1 = 2*i-1;
    pp2 = 2*i;
    
    t = (i-1) / (num_profiles-1);
    
    [p, p1, p2] = point_at(t, xs, ys);
    ps(i, :) = p;   
    
%   Normal to line segment
    v = p2 - p1;
    n = [-v(2) v(1)];
    n = n / norm(n);  
    
    prof_ps(pp1, :) = p + prof_length/2 * n;
    prof_ps(pp2, :) = p - prof_length/2 * n;
end

end