%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This work is licensed under a Creative Commons Attribution 4.0
% International License. http://creativecommons.org/licenses/by/4.0/
% The authors do not assume liability for errors contained in or for
% damages arising from the use of the software.
%
% CC-BY (4.0) Oliver Taubmann (oliver.taubmann@fau.de), 2015
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [p, p1, p2] = point_at(t, xs, ys)
% For the line segments defined in (xs, ys), get the point p
% at position t=[0..1] along the complete line. p1 and p2 are
% the start and end points of the corresponding segment

lens    = sqrt((xs(2:end) - xs(1:end-1)).^2 + (ys(2:end) - ys(1:end-1)).^2);
len     = sum(lens);
cumlens = cumsum(lens);

t_len = len * t;

if (t_len < cumlens(1))
    seg_idx = 1;
    t_local = t_len / lens(seg_idx);
else
    seg_idx = find(cumlens >= t_len);
    seg_idx = seg_idx(1);
    t_local = (t_len-cumlens(seg_idx-1)) / lens(seg_idx);
end

p1 = [xs(seg_idx  ) ys(seg_idx  )];
p2 = [xs(seg_idx+1) ys(seg_idx+1)];

p = t_local * p2 + (1 - t_local) * p1;

end