%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This work is licensed under a Creative Commons Attribution 4.0
% International License. http://creativecommons.org/licenses/by/4.0/
% The authors do not assume liability for errors contained in or for
% damages arising from the use of the software.
%
% CC-BY (4.0) Oliver Taubmann (oliver.taubmann@fau.de), 2015
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function profs = sample_profiles(prof_ps, slice, num_samples)
% Samples profiles in prof_ps on slice
% Number of samples is an optional parameter

if ~exist('num_samples', 'var') || isempty(num_samples)
  num_samples = 40;
end

profs = zeros(size(prof_ps, 1) / 2, num_samples);

% Line profile

for i = 1:size(profs, 1)
    pp1 = 2*i-1;
    pp2 = 2*i;
    ts = linspace(0, 1, num_samples)';
    samples = (1 - ts) * prof_ps(pp1, :) + ts * prof_ps(pp2, :);
    profs(i, :) = interp2(slice, samples(:, 1), samples(:, 2));
end

end